#include <libgtherm.h>

int
main (int    argc,
      char **argv)
{
  gth_manager_new_sync (NULL,
                        G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START,
                        NULL,
                        NULL);
}
